<html>
<head>
<style type="text/css">
pre {
	font-family: Tahoma,Arial,Helvetica,sans-serif;
	font-size: 11px;
}
</style>
<body>
<pre>

<?php
/* Basic introduction to SQL.php
*/
 
 
/**** INITIALIZATION *********************************************************/
 
// Configure the database settings
define("DB_NAME","testdb");		// database name
define("DB_USER","testdb");		// database username
define("DB_PASS","testdb");		// database password
define("DB_HOST","localhost");	// database hostname

// include the class
require_once("sql.php");

// instantiate the class
$sql = &new SQL();



/**** INSERT QUERIES *********************************************************/
		
// Normally your input would come from a form or somewhere similar, but this is just
// an example
$name = "Test Item #".rand(0,4000);
$price = rand(1,100)+0.99;

// ALWAYS ESCAPE USER INPUT USING $sql->safestr() PRIOR TO USING IT IN QUERIES!
// Do not use addslashes().  $sql->safestr() is a wrapper for mysql_escape_string().
$name = $sql->safestr($name);
$price = $sql->safestr($price);

// insert_query() performs the specified INSERT query, and then returns the
// ID number of the newly created row.
$query = "INSERT INTO test_items (name,price) VALUES ('$name','$price')";
$new_product_id = $sql->insert_query($query);
if ($new_product_id===false) die("Error: ".$sql->db_error);


echo "\n\nCreated product #".$new_product_id." via insert_query()\n";


/**** AUTOMATED INSERT QUERIES ***********************************************/

// A quicker (and more foolproof) way to perform insert queries is to put them
// into an array, then let the SQL class build the INSERT statement.
//
// This way, your user input is escaped automatically by the SQL class - you
// don't need to do it yourself.

$name = "Test Item #".rand(0,4000);
$price = rand(1,100)+0.99;

// the array KEYS below (name and price) are the database column names, the 
// array VALUES below are the values that will be inserted into those columns.
$values = array(
	"name"=>$name,
	"price"=>$price
);

// insert the array items into the test_items table
$new_product_id = $sql->insert_array_query("test_items",$values);
if ($new_product_id===false) die("Error: ".$sql->db_error);

echo "\n\nCreated product #".$new_product_id." via insert_array_query()\n";



/**** UPDATE QUERIES *********************************************************/

// misc_query() performs a query and returns TRUE on success, FALSE on failure

$itemid_to_update = 1;
$newname = "Updated Product #".rand(1,4000);
$newname = $sql->safestr($newname);

$query = "UPDATE test_items SET name='$name' WHERE id=$itemid_to_update";
$res = $sql->misc_query($query);

if ($res===false) die("Error: ".$sql->db_error);



/**** AUTOMATED UPDATE QUERIES ***********************************************/

// A quicker (and more foolproof) way to perform update queries is to put them
// into an array, then let the SQL class build the UPDATE statement.
//
// This way, your user input is escaped automatically by the SQL class - you
// don't need to do it yourself.

$itemid_to_update = 1;
$newname = "Updated Product #".rand(1,4000);
$values = array(
	"name"=>$newname
);

// to update an item WHERE id=$itemid_to_update, use:
$sql->update_array_query("test_items",$values,$itemid_to_update);
if ($res===false) die("Error: ".$sql->db_error);

// to update an item with a more complex WHERE clause, use:
$whereclause = "(id=$itemid_to_update) OR (price>100.00)";
$sql->update_array_query("test_items",$values,NULL,NULL,NULL,$whereclause);
if ($res===false) die("Error: ".$sql->db_error);



/**** SELECT QUERIES (MULTIPLE RESULT ROWS) **********************************/

// select_all() returns an array containing ALL of the items returned by the
// query; this saves you from having to run repeated mysql_fetch_row() calls
$query = "SELECT * FROM test_items ORDER BY id DESC";
$items = $sql->select_all($query);

// If any SQL class method returns FALSE, a database error occurred
if ($items===false) die("Error: ".$sql->db_error);

echo "\n\nQuery via select_all() returned:\n";
var_dump($items);



/**** SELECT QUERIES (SINGLE RESULT ROW) *************************************/

// select_query() performs the query and returns a single result row.
$query = "SELECT SUM(price) AS total FROM test_items";
$res = $sql->select_query($query);

if ($res===false) die("Error: ".$sql->db_error);


echo "\n\nTotal value of all items:\n";
var_dump($res);

?>

</pre>
</body>
</html>