<?php
// MySQL Database Abstraction Class
// Version 1.5.0
// Copyright 2003-2005, Steve Blinch
// http://www.esitemedia.com
// ============================================================================
// 
// LICENSE
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//
// NOTES
//
// Define SQL server connection defaults using constants below.
// MUST BE SET BEFORE USING THIS CLASS!
//
// define("DB_HOST","localhost");
// define("DB_USER","root");
// define("DB_PASS","password");
// define("DB_NAME","database");

class SQL {

	// SQL connection settings
	var $db_host = DB_HOST;
	var $db_user = DB_USER;
	var $db_pass = DB_PASS;
	var $db_name = DB_NAME;
	var $tbl_cat = TBL_CAT;
	var $key_names = KEY_NAMES;

	// Database connection
	var $db;
	
	// Internal result ID; do not use
	var $db_result;
	
	// If a function returns FAILED, the error message will be stored here
	var $db_error;
	
		
	function SQL($new_name="",$new_host="",$new_user="",$new_pass="",$new_link=false) {
		//
		// Constructor
		//
		$this->supports_newlink = (version_compare("4.2.0",phpversion())<=0);

		$do_init=1;
		if ($new_host) { $this->db_host=$new_host; $do_init=1; }
		if ($new_user) { $this->db_user=$new_user; $do_init=1; }
		if ($new_pass) { $this->db_pass=$new_pass; $do_init=1; }
		if ($new_name) { $this->db_name=$new_name; $do_init=1; }
		if ($do_init) { $this->init($new_link); }
	}
	
	function safestr($s,$striphtml=false) {
		if ($striphtml) $s = strip_tags($s);
		return mysql_escape_string($s);
	}
	
	function safearr($a) {
		for ($i=0; $i<count($a); $i++) {
			$a[$i] = mysql_escape_string($a[$i]);
		}
		
		return $a;
	}
	
	function init($new_link=false) {
		//
		// Initializes the Categories instance.
		//
		// At the moment, this just connects to the SQL server and sets the active database.
		//
		$this->connect_database($new_link);
	}
	
	function close_connection() {
		mysql_close($this->db);
		$this->connected = false;
	}
	
	function connect_database($new_link=false) {
		//
		// Connects to the SQL server and sets the active database.
		//
		$this->connected = 1;
		if ($this->supports_newlink) {
			$this->db = @mysql_connect($this->db_host, $this->db_user, $this->db_pass, $new_link) or $this->connected=0;
		} else {
			$this->db = @mysql_connect($this->db_host, $this->db_user, $this->db_pass) or $this->connected=0;
		}
		if (!$this->connected) $this->set_error(mysql_error());
		$this->set_database();
	}
	
	
	function table_exists($tablename) {
		$tablename = $this->safestr($tablename);
		$query = "SHOW tables LIKE '$tablename'";
		$tables = $this->select_query($query);
		return $tables?true:false;
	}
	
	
	function log_query($sql) {
/*
		$fp = fopen("/tmp/sql_log","a+");
		if ($fp) {
			fwrite($fp,"$sql\n");
			fclose($fp);
		}
*/
	}
	
	function set_database($new_db="") {
		//
		// Sets the active database.  If new_db is specified, the active database is set to it.
		// If not, it uses the current this->db_name.
		//
		if ($new_db) { $this->db_name = $new_db; }
		if ($this->connected) { mysql_select_db($this->db_name,$this->db); }
	}

	function set_error($s) {
		//
		// Called internally to set the error message generated by a failed method call.
		//
		if (!$this->suppress_errors) echo "<p><font color='#CC0000'>[</font><font color='#FF0000'>SQLERR</font><font color='#CC0000'>]</font>: ".htmlentities($s)."</p>\n";
		
		$this->db_error = $s;
	}
	
	function insert_query($sql) {
		//
		// Wrapper for mysql_query(), for use with "INSERT INTO" queries.
		//
		// Returns the ID of the new row on success, or FALSE on error.
		//
		$this->log_query($sql);
		$result = @mysql_query($sql,$this->db) or $this->set_error("Could not execute query: $sql: ".mysql_error($this->db));
		if ($result) {
			$this->affected = mysql_affected_rows($this->db);
			return mysql_insert_id($this->db);
		} else {
			return $result; 
		}
	}

	function misc_query($sql) {
		//
		// Wrapper for mysql_query(), for use with miscellaneous queries.
		//
		// Basically just for consistency with insert_query and() select_query().
		// Returns the TRUE on success, FALSE on failure.
		//
		$this->log_query($sql);
		$result = (@mysql_query($sql,$this->db) or $this->set_error("Could not execute query: $sql: ".mysql_error($this->db)));
		$this->affected = mysql_affected_rows($this->db);
		return $result;
		
	}
	
	function select_query($sql,$free_on_return=false) {
		//
		// Wrapper for mysql_query(), for use with SELECT queries.
		//
		// Returns the first result row on success, or FALSE on failure.
		// Subsequent rows may be retrieved using select_next() or next_result().
		//
		$this->log_query($sql);
		$result = @mysql_query($sql,$this->db) or $this->set_error("Could not execute query: $sql: ".mysql_error($this->db));
		if ($result) {
			$this->db_result=$result;
			$this->results = mysql_num_rows($result);

			$res = $this->select_next();
			if ($free_on_return) $this->free_result();
			return $res;
		} else {
			return $result; 
		}
	}

	function free_result() {
		if ($this->db_result) @mysql_free_result($this->db_result);
	}

	function select_next() {
		//
		// Wrapper for mysql_fetch_array().
		//
		// Automatically strips escape characters (slashes) from string-type elements
		// prior to returning.
		// Returns the next result row on success, or FALSE on failure.
		//
		if ($myrow = mysql_fetch_array($this->db_result)) {
			$k = array_keys($myrow);
			for ($i=0; $i<count($myrow); $i++) {
				if (is_string($myrow[$k[$i]])) {
					$myrow[$k[$i]]=stripslashes($myrow[$k[$i]]);
				}
			}
		}
		return $myrow;
	}
	
	function select_all($sql,$byid=false,$groupby=NULL,$groupby_legacy_compatible=true) {
		//
		// Executes a select query and returns ALL result rows for that query.
		// If $byid is true, an associative array is returned, keyed by the row ID.
		//
		// If $groupby is not NULL, rows are grouped by the $groupby column;
		// that is, the result array will contain one or more sub-arrays keyed
		// with the values of the $groupby columns.
		//
		// $groupby_legacy_compatible is for compatibility with older apps, in which
		// results were returned as $res[$groupby_col][0]=$value; disabling compatibility
		// mode returns results as $res[$groupby_col]=$value.
		// 
		$output = array();
		$res = $this->select_query($sql);
		if (!is_array($res)) { return false; }
		while (is_array($res)) {
			if (!is_null($groupby)) {
				if ($groupby_legacy_compatible) {
					$output[$res[$groupby]][] = $res;
				} else {
					$output[$res[$groupby]] = $res;
				}
			} elseif ($byid) {
				$output[$res["id"]] = $res;
			} else {
				$output[] = $res;
			}
			$res = $this->select_next();
		}
		$this->free_result();
		return $output;
	}

	function next_result() {
		//
		// same as select_next()
		//
		return $this->select_next();
	}
	
	function list_fields($tbl_name) {
		//
		// Returns all fields in the specified table.
		// 
		$fields = mysql_list_fields($this->db_name,$tbl_name,$this->db);
		$columns = mysql_num_fields($fields);

		$res = array();
		for ($i = 0; $i < $columns; $i++) {
  		$res[] = mysql_field_name($fields, $i);
		}
		return $res;
	}
	
	function timestamp_to_english($timestamp,$shortmonths=false,$dateonly=false,$timeonly=false,$simple=false) {
		//
		// Converts a MySQL timestamp value to an english date/time
		// 
		$mos = array("January","Feburary","March","April","May","June","July","August","September","October","November","December");

		$year = (int) substr($timestamp,0,4);
		$month = (int) substr($timestamp,4,2);
		$day = (int) substr($timestamp,6,2);
		$hour = substr($timestamp,8,2);
		$min = substr($timestamp,10,2);
		$sec = substr($timestamp,12,2);
		
		$mo = $mos[$month-1];
		if ($shortmonths) { $mo = substr($mo,0,3); }
		if ($simple) {
			$date = "$day $mo $year";
			$time = "$hour:$min:$sec";
		} else {
			$date = "$mo $day, $year";
			$time = "$hour:$min:$sec";
		}
		if ($dateonly) { 
			return $date;
		} elseif ($timeonly) {
			return $time;
		} else {
			return "$date $time";
		}
	}
	
	function get_cols_vals($arr,&$cols,&$vals,$safe=true,$restrict_keys=NULL) {
		if (is_array($arr)) {
			reset($arr);
			foreach ($arr as $name=>$value) {
				if (is_array($restrict_keys)) {
					if (!in_array($name,$restrict_keys)) continue;
					if (is_numeric($name)) continue;
				}
				
				$cols .= $name.",";
				if ($safe) {
					$vals .= "'".$this->safestr($value)."',";
				} else {
					$vals .= $value.",";
				}
			}
			$cols = substr($cols,0,strlen($cols)-1);
			$vals = substr($vals,0,strlen($vals)-1);
		}
	}

	function insert_array_query($table,$values,$literals=NULL,$restrict_keys=NULL) {
		//
		// Inserts values into a table directly from an array
		//
		// $table is the table to insert into; $values is an array of
		// fieldname=>value pairs to insert into the table.
		//
		// $literals is an additional array of fieldname=>value pairs to insert,
		// however "value" is not escaped (so it may contain MySQL commands such
		// as "NOW()", etc.)
		//
		// If $restrict_keys is non-null, it must specify a list of valid field
		// names to pull from values; any unspecified fields will be ignored.
		//
		// Returns non-zero on success, false on failure.
		// 
		$this->get_cols_vals($values,$cols,$vals,true,$restrict_keys);
		if (is_array($literals)) {
			$this->get_cols_vals($literals,$cols_l,$vals_l,false);
			if ($cols_l) {
				$cols .= ",$cols_l";
				$vals .= ",$vals_l";
			}
		}
		
		$query = "INSERT INTO $table ($cols) VALUES ($vals)";
		$id = $this->insert_query($query);
		return $id;
	}

	function update_array_query($table,$values,$id,$literals=NULL,$restrict_keys=NULL,$whereclause=NULL) {
		//
		// Updates values in a table directly from an array
		//
		// $table is the table to update; $values is an array of
		// fieldname=>value pairs to insert into the table; $id
		// is the ID number to update.
		//
		// Returns non-zero on success, false on failure.
		// 
		reset($values);
		foreach ($values as $name=>$value) {
			if (is_array($restrict_keys)) {
				if (!in_array($name,$restrict_keys)) continue;
				if (is_numeric($name)) continue;
			}
			$colvals .= "$name='".$this->safestr($value)."',";
		}
		$colvals = substr($colvals,0,strlen($colvals)-1);
		
		if (is_array($literals)) {
			reset($literals);
			foreach ($literals as $name=>$value) {
				$colvals_l .= "$name=$value,";
			}
			if ($colvals_l) {
				$colvals .= ",".substr($colvals_l,0,strlen($colvals_l)-1);
			}
		}
		
		$query = "UPDATE $table SET $colvals".($id==NULL?"":" WHERE id=".((int) $id)).($whereclause==NULL?"":" WHERE $whereclause");
		return $this->misc_query($query);
	}
	
	function update_request_query($table,$fields,$id,$literals=NULL) {
		$values = array();
		foreach ($fields as $k=>$field) {
			$values[$field] = $_REQUEST[$field];
		}
		return $this->update_array_query($table,$values,$id,$literals);
	}

	function insert_request_query($table,$fields,$extrafields=NULL,$literals=NULL) {
		//
		// Inserts values into a table directly from the $_REQUEST array.
		//
		// $table is the table to insert into; $fields is an array of
		// field names to insert.  Field names MUST match the keys in
		// the $_REQUEST array.
		//
		// Returns non-zero on success, false on failure.
		// 
		reset($fields);
		foreach ($fields as $k=>$name) {
			$cols .= $name.",";
			$vals .= "'".$this->safestr($_REQUEST[$name])."',";
		}
		if (is_array($extrafields)) {
			foreach ($extrafields as $name=>$value) {
				$cols .= $name.",";
				$vals .= "'".$this->safestr($value)."',";
			}			
		}

		$cols = substr($cols,0,strlen($cols)-1);
		$vals = substr($vals,0,strlen($vals)-1);
		
		if (is_array($literals)) {
			$this->get_cols_vals($literals,$cols_l,$vals_l,false);
			if ($cols_l) {
				$cols .= ",$cols_l";
				$vals .= ",$vals_l";
			}
		}
		

		$query = "INSERT INTO $table ($cols) VALUES ($vals)";
		return $this->insert_query($query);
	}
	
	function keys_to_vals(&$arr) {
		$res = array();
		foreach ($arr as $k=>$v) {
			$res[] = $k;
		}
		return $res;
	}
	
	function array_stripslashes($arr) {
		foreach ($arr as $k=>$v) {
			if (is_string($v)) {
				$arr[$k] = stripslashes($v);
			} elseif (is_array($v)) {
				$arr[$k] = $this->array_stripslashes($v);
			}
		}
		return $arr;
	}

}

?>
