<?php
/* This is an example of how to use the SQL class, but it also shows the
 * file path layout I usually use (eg: an include.php which includes the
 * configuration file, then instantiates the SQL and Template classes).
 *
 * For a less complex introduction to SQL.php, please see the example_basic
 * directory.
 */

require_once(dirname(__FILE__)."/includes/include.php");
$tpl->display("header.tpl");

// if the user is submitting an item, insert it into the database
if ($_REQUEST["additem"]) {
	
	
	$manual_insert = true;
	
	// There are two ways to perform database inserts.  One is to manually escape
	// each item and type in an INSERT query yourself, using $sql->insert_query().
	//
	// The other is to use a pre-made function which automatically escapes all of
	// your variables and builds the INSERT query for you, using $sql->insert_array_query().

	if ($manual_insert) {
		// METHOD ONE: USING $sql->insert_query()
		
		// ALWAYS ESCAPE USER INPUT USING $sql->safestr() PRIOR TO USING IT IN QUERIES!
		// Do not use addslashes().  $sql->safestr() is a wrapper for mysql_escape_string().
		$name = $sql->safestr($_REQUEST["name"]);
		$price = $sql->safestr($_REQUEST["price"]);
		
		// Insert an item into the database.
		//
		// insert_query() performs the specified INSERT query, and then returns the
		// ID number of the newly created row.
		$query = "INSERT INTO test_items (name,price) VALUES ('$name','$price')";
		$new_product_id = $sql->insert_query($query);
		
		if ($new_product_id===false) die("Error: ".$sql->db_error);
	
	} else {
		// METHOD TWO: USING $sql->insert_array_query();
		
		// the array KEYS below are the database column names, the array VALUES below
		// are the values that will be inserted into those columns.
		$values = array(
			"name"=>$_REQUEST["name"],
			"price"=>$_REQUEST["price"]
		);
		
		// insert the array items into the test_items table
		$new_product_id = $sql->insert_array_query("test_items",$values);
		if ($new_product_id===false) die("Error: ".$sql->db_error);
		
		// As you can see, this way is a lot simpler, and it automatically escapes the
		// variables for you.
	}
	
}


// Load the products list.
//
// select_all() returns an array containing ALL of the items returned by the
// query; this saves you from having to run repeated mysql_fetch_row() calls
$query = "SELECT * FROM test_items ORDER BY id DESC";
$products = $sql->select_all($query);

// if select_all() (or any of the other SQL class methods) returns false, this 
// means that an SQL server error occurred; the error message will be stored
// in $sql->db_error
if ($products===false) die("Error: ".$sql->db_error);

$tpl->assign("products",$products);

// Get the total cost of all products in the database.
//
// select_query() performs the query and returns a single result row.
$query = "SELECT SUM(price) AS total FROM test_items";
$res = $sql->select_query($query);

$tpl->assign("totalvalue",$res["total"]);

$tpl->display("index.tpl");

$tpl->display("footer.tpl"); 


// Although not demonstrated here, you can also use $sql->misc_query() for 
// miscellaneous queries such as UPDATE, CREATE, and ALTER queries.
//
// misc_query() returns TRUE on success, FALSE on error.
//
//
// There is also an $sql->update_array_query() method, which is similar to
// $sql->insert_array_query() except that it builds an UPDATE query.
//
// When in doubt, please refer to SQL.php - each method is commented there.

?>
