<?php
/* MySQL Database Abstraction Class
 * Version 1.5.7
 * Copyright 2003-2007, Steve Blinch
 * http://code.blitzaffe.com
 * ============================================================================
 *
 * DESCRIPTION
 *
 * Sometimes ADODB is overkill for small projects, and a lighter, faster
 * SQL library comes in handy.
 *
 * This library provides a convenient wrapper for the MySQL database functions
 * provided by PHP.
 *
 *
 * CHANGELOG
 *
 * 1.5.7 (15-Oct-2007)
 *			- Added UTF-8 support
 *			- Added class to SQL errors to allow styling
 *			- Made _backtrace_args() a static method
 *			- Added multiple-row-insert support to insert_array_query()
 *
 * 1.5.6 (11-Oct-2006)
 *			- Fixed bug in array_stripslashes() when magic_quotes_gpc is disabled
 *
 * 1.5.5 (19-Jun-2006)
 *			- Added constants for "too many connections" detection
 *			- Added singleton() class method
 *			- Added get_database() method
 *			- Added $debugqueries class variable
 *
 * 1.5.4 (18-Apr-2006)
 *			- Changed default behavior for select_query() to automatically free
 *			  the result resource
 *			- Added support for reserved keywords as column names in the array-
 *			  based query methods
 *			- Added optional support for stripping slashes for values passed
 *			  to the array-based query methods
 *
 * 1.5.3 (Unknown)
 *			- Added support for plaintext error messages (eg: for use in
 *			  commandline scripts)
 *			- Added support for custom error handlers via $sql->error_handler
 *
 * 1.5.2 (13-Dec-2005)
 *			- Added much more comprehensive error handling
 *			- Added support for backtraces upon error
 *
 *
 * 
 * LICENSE
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * NOTES
 * 
 * Define SQL server connection defaults using constants below.
 * THESE MUST BE SET BEFORE USING THIS CLASS, unless you plan to pass
 * these values to the constructor.
 * 
 * define("DB_HOST","localhost");
 * define("DB_USER","root");
 * define("DB_PASS","password");
 * define("DB_NAME","database");
 */

define('SQL_NO_STRIPSLASHES',true);
define('SQL_FREE_ON_RETURN',true);

define('MYSQL_ER_TOO_MANY_USER_CONNECTIONS',1203);
define('MYSQL_ER_CON_COUNT_ERROR',1040);

class SQL {

	// SQL connection settings
	var $db_host = DB_HOST;
	var $db_user = DB_USER;
	var $db_pass = DB_PASS;
	var $db_name = DB_NAME;
	var $tbl_cat = TBL_CAT;
	var $key_names = KEY_NAMES;

	// Database connection
	var $db;
	
	// Internal result IDs; do not use
	var $db_result;
	var $db_results_stored;
	
	// If a function returns FAILED, the error message will be stored here
	var $db_error;
	var $error_message;
	
	// If a MySQL error occurs, the code will be stored here	
	var $error_code;
	
	var $logfile = false;
	

	// If TRUE, detailed SQL error messages will be replaced with simple, non-descriptive
	// messages indicating that a nonspecific error has occurred.
	var $simple_errors = false;

	// If TRUE, all error messages will be entirely suppressed.
	var $suppress_errors = false;	
	
	// If TRUE, a backtrace will be printed after all errors.
	var $backtrace_errors = false;
	
	// If TRUE, error output will be in plaintext rather than HTML.
	var $plaintext_errors = false;
	
	// If set to a valid callback, will be called whenever an SQL error occurs
	var $error_handler = null;
	
	// If TRUE, strip slashes from values prior to use in the insert_array_query,
	// update_array_query, insert_request_query, or update_request_query methods
	var $stripslashes_before_query = false;
	
	// If TRUE, all queries will be stored in $this->queries
	var $debugqueries = false;
	
	var $utf8 = false;
		
	function SQL($new_name="",$new_host="",$new_user="",$new_pass="",$new_link=false,$utf8 = false) {
		//
		// Constructor
		//
		$this->utf8 = (defined('SQL_UTF8') && SQL_UTF8) || $utf8;
		$this->supports_newlink = (version_compare("4.2.0",phpversion())<=0);
		$this->fetch_method = defined('SQL_FETCH_METHOD') ? SQL_FETCH_METHOD : MYSQL_BOTH;
		if (defined('SQL_PLAINTEXT') && SQL_PLAINTEXT) $this->plaintext_errors = true;
		
		$do_init=1;
		if ($new_host) { $this->db_host=$new_host; $do_init=1; }
		if ($new_user) { $this->db_user=$new_user; $do_init=1; }
		if ($new_pass) { $this->db_pass=$new_pass; $do_init=1; }
		if ($new_name) { $this->db_name=$new_name; $do_init=1; }
		if ($do_init) { $this->init($new_link); }
	}
	
	function check_connected() {
		$this->connected = is_resource($this->db);
		if (!$this->connected) return false;
		
		if (version_compare("4.3.0",phpversion())<=0) { 
			$this->connected = mysql_ping($this->db);
		}
		
		return $this->connected;
	}
	
	function safestr($s,$striphtml=false) {
		if ($striphtml) $s = strip_tags($s);
		return mysql_escape_string($s);
	}
	
	function safearr($a) {
		for ($i=0; $i<count($a); $i++) {
			$a[$i] = mysql_escape_string($a[$i]);
		}
		
		return $a;
	}
	
	function init($new_link=false) {
		//
		// Initializes the Categories instance.
		//
		// At the moment, this just connects to the SQL server and sets the active database.
		//
		$this->connect_database($new_link);
	}
	
	function close_connection() {
		mysql_close($this->db);
		$this->connected = false;
	}
	
	function connect_database($new_link=false) {
		//
		// Connects to the SQL server and sets the active database.
		//
		$this->connected = 1;
		if ($this->supports_newlink) {
			$this->db = @mysql_connect($this->db_host, $this->db_user, $this->db_pass, $new_link) or $this->connected=0;
		} else {
			$this->db = @mysql_connect($this->db_host, $this->db_user, $this->db_pass) or $this->connected=0;
		}
		if ($this->connected) {
			if ($this->utf8) $this->misc_query('SET NAMES "utf8"');
		} else {
			$this->set_error('Could not connect to database','',mysql_error(),mysql_errno());
		}
		if (!$this->set_database()) $this->connected = false;
	}
	
	
	function table_exists($tablename) {
		$tablename = $this->safestr($tablename);
		$query = "SHOW tables LIKE '$tablename'";
		$tables = $this->select_query($query);
		$this->free_result();
		return $tables?true:false;
	}
	
	
	function log_query($sql) {
		if ($this->logfile) {
			$fp = fopen($this->logfile,"a+");
			if ($fp) {
				fwrite($fp,"$sql\n");
				fclose($fp);
			}
		}
	}
	
	function query_debug_start($query) {
		if (!$this->debugqueries) return;
		
		$this->active_query = array(
			'start'=>$this->get_microtime(),
			'query'=>$query
		);
	}
	
	function query_debug_error($error) {
		if (!$this->debugqueries) return;

		$this->active_query['error'] = $error;
	}

	function query_debug_stop() {
		if (!$this->debugqueries) return;

		$this->active_query['time'] = $this->get_microtime() - $this->active_query['start'];

		$this->queries[] = $this->active_query;
		unset($this->active_query);
	}
	
	function query_debug_continue() {
		if (!$this->debugqueries) return;
		
		$this->active_query = array_pop($this->queries);
	}
	
	function get_microtime() {
		list($usec, $sec) = explode(" ", microtime());
		return ((float)$usec + (float)$sec);
	}
		
	
	function set_database($new_db="") {
		//
		// Sets the active database.  If new_db is specified, the active database is set to it.
		// If not, it uses the current this->db_name.
		//
		if ($new_db) { $this->db_name = $new_db; }
		if ($this->connected) { 
			$ok = mysql_select_db($this->db_name,$this->db); 
			if (!$ok) $this->set_error(sprintf('Could not select database: %s',$this->db_name),'',mysql_error(),mysql_errno());
			return $ok;
		} else {
			return false;
		}
	}

	function get_database() {
		$query = "SELECT DATABASE() AS dbname";
		$res = $this->select_query($query);
		
		return $res['dbname'];
	}
	
	function _backtrace_args($arglist) {
		if (!is_array($arglist)) return "";
		foreach ($arglist as $k=>$arg) {
			$args .= gettype($arg);
			
			if (is_object($arg)) {
				$args .= ' '.get_class($arg);
			} elseif (is_string($arg)) {
				if (strlen($arg)>30) $arg = substr($arg,0,30).'...';
				$args .= '="'.$arg.'"';
			} elseif (is_array($arg)) {
				$args .= '('.count($arg).')';
			} elseif (is_bool($arg)) {
				$args .= '='.($arg?'true':'false');
			} elseif (is_scalar($arg)) {
				$args .= '='.$arg;
			}
			$args .= ",";
		}
		return substr($args,0,strlen($args)-1);
	}

	function set_error($description,$errquery,$sqlerror,$errorcode) {
		//
		// Called internally to set the error message generated by a failed method call.
		//
		
			
		if ($this->suppress_errors) {
		} elseif ($this->simple_errors) {
			$fmt = $this->plaintext_errors ? '%s' : "<p style='color: #FF0000; padding: 8px'>%s</p>";

			$error = sprintf($fmt,"A database error has occurred.");
		} else {
			if (function_exists('debug_backtrace')) {
				$backtrace = debug_backtrace();
				array_shift($backtrace); // skip set_error call
				
				$sql_call = array_shift($backtrace); // get mysql API call
				$user_call = $sql_call;
				
				while ($user_call['file']==__FILE__) {
					$user_call = array_shift($backtrace);
				}
				
				if (!$errquery) $errquery = '(N/A)';
		
				if ($this->error_handler && is_callable($this->error_handler)) {
					call_user_func(
						$this->error_handler,
						array(
							'description'=>$description,
							'file'=>$user_call['file'], 
							'line'=>$user_call['line'],
							'query'=>trim($errquery),
							'error'=>$sqlerror,
							'class'=>$sql_call['class'],
							'type'=>$sql_call['type'],
							'function'=>$sql_call['function'],
							'args'=>$this->_backtrace_args($sql_call['args'])
						)
					);
					$error = "";
				} elseif ($this->plaintext_errors) {
					
					$error = sprintf(
						str_repeat('=',79) . "\n" .
						"Error:\t%s\n" .
						"File:\t%s:%d\n" .
						"Query:\t%s\n" .
						"Reason:\t%s\n" .
						"Call:\t\$%s%s%s(%s)\n" .
						str_repeat('=',79) . "\n",
						$description,
						$user_call['file'], $user_call['line'],
						trim($errquery),
						$sqlerror,
						$sql_call['class'],
							$sql_call['type'],
							$sql_call['function'],
							$this->_backtrace_args($sql_call['args'])
					);
										
				} else {
					
					$errorfont = "font-family: Tahoma,Arial; font-size: 11px";
					$namestyle = "font-weight: bold; $errorfont";
					$valuestyle = "background-color: #FFFFFF; border: 1px dashed #E0E0E0; padding: 2px 5px 2px 5px; $errorfont";
					
					$error = sprintf(
						'<table cellspacing="0" cellpadding="2" style="background-color: #F0F0F0; padding: 3px; margin: 8px; border: 1px solid #E0E0E0;" class="sqlerror">'.
							'<tr valign="top"><td style="padding-bottom: 5px; '.$errorfont.'" colspan="2"><strong style="color: #FF0000;">SQL Error:</strong> %s in <strong>%s</strong>:%d</td>'.
							'<tr valign="top"><td style="%s">Query:&nbsp;	</td><td style="%s; border-bottom: none">%s</td></tr>'.
							'<tr valign="top"><td style="%s">Error:			</td><td style="%s; border-bottom: none">%s</td></tr>'.
							'<tr valign="top"><td style="%s">Call:			</td><td style="%s; border-bottom: none">$%s%s<strong>%s</strong>(<span style="color: #888888">%s</span>)</td></tr>'.
						'</table>',
						htmlentities($description),
						htmlentities($user_call['file']), $user_call['line'],
						$namestyle,$valuestyle, htmlentities($errquery),
						$namestyle,$valuestyle, htmlentities($sqlerror),
						$namestyle,$valuestyle, $sql_call['class'],
							htmlentities($sql_call['type']),
							$sql_call['function'],
							htmlentities($this->_backtrace_args($sql_call['args']))
					);
				}

			} else {
				$error = sprintf(
					"<p style='padding: 8px'><span style='color: #CC0000'>[<span style='color: #FF0000'>SQL Error</span>]</span>: %s: %s: %s</p>\n",
					$description,$errquery,$sqlerror
				);
				if ($this->error_handler && is_callable($this->error_handler)) {
					call_user_func(
						$this->error_handler,
						array(
							'description'=>$description,
							'query'=>trim($errquery),
							'error'=>$sqlerror,
						)
					);
					$error = "";
				} elseif ($this->plaintext_errors) {
					$error = strip_tags($error);
				}
				
			}
			
			echo $error;
			
			if ($this->backtrace_errors) $this->print_backtrace();
		}

		
		$this->error_code = $errorcode;
		$this->error_message = $this->db_error = $description . ': ' . $sqlerror;
		$this->log_query("ERROR: $errmsg");

		if ($this->debugqueries) $this->query_debug_error("ERROR: $errmsg");
	}
	
	function print_backtrace() {
	
		$include_functions = array('include','require','include_once','require_once');
		
		echo "<div style='background-color: #F5F5F5; border: 1px solid #D0D0D0; padding: 8px; font-family: Courier; font-size: 11px'>";
	
		if (!function_exists('debug_backtrace')) {
			echo "Backtrace not supported on this PHP build</div>";
			return false;
		}
		
		$bt = debug_backtrace();
		
		$info = $bt[0];
		
		echo sprintf(
			"<span style='color: #888888'>Backtrace requested by <strong>%s</strong>:%d</span><br /><br />",
			$info['file'],
			$info['line']
		);
		
		foreach ($bt as $k=>$item) {
			$args = '';
			$args = SQL::_backtrace_args($item['args']);
			
			if (in_array($item['function'],$include_functions)) {
				$args = $item['args'][0];
			}
			
			echo sprintf(
				"Called <strong style='color: #FF9900'>%s</strong>%s<strong style='color: #990000'>%s</strong>(<span style='color: #008800'>%s</span>) from <strong style='color: #000099'>%s</strong>:%d<br />",
				( strlen($item['class']) && ($item['type']=='->') ) ? '$'.$item['class'] : $item['class'],
				$item['type'],
				$item['function'],
				$args,
				$item['file'],
				$item['line']
			);
		}
		
		echo sprintf(
			"<br /><span style='color: #888888'>Start</span>",
			$info['file'],
			$info['line']
		);
		
		echo "</div>";
	}
	
	function insert_query($sql) {
		//
		// Wrapper for mysql_query(), for use with "INSERT INTO" queries.
		//
		// Returns the ID of the new row on success, or FALSE on error.
		//
		
		if ($this->debugqueries) $this->query_debug_start($sql);
		
		$this->log_query($sql);
		$result = @mysql_query($sql,$this->db) or $this->set_error('Could not execute query',$sql,mysql_error($this->db),mysql_errno($this->db));
		if ($result) {
			$this->affected = mysql_affected_rows($this->db);
			$insert_id = mysql_insert_id($this->db);
			
			if ($this->debugqueries) $this->query_debug_stop();
			
			return $insert_id;
		} else {
			if ($this->debugqueries) $this->query_debug_stop();

			return $result; 
		}
	}

	function misc_query($sql) {
		//
		// Wrapper for mysql_query(), for use with miscellaneous queries.
		//
		// Basically just for consistency with insert_query and() select_query().
		// Returns the TRUE on success, FALSE on failure.
		//
		if ($this->debugqueries) $this->query_debug_start($sql);

		$this->log_query($sql);
		$result = (@mysql_query($sql,$this->db) or $this->set_error('Could not execute query',$sql,mysql_error($this->db),mysql_errno($this->db)));
		$this->affected = mysql_affected_rows($this->db);

		if ($this->debugqueries) $this->query_debug_stop();
		
		return $result;
		
	}
	
	function select_query($sql,$free_on_return=SQL_FREE_ON_RETURN,$no_stripslashes=SQL_NO_STRIPSLASHES,$queryid = false) {
		//
		// Wrapper for mysql_query(), for use with SELECT queries.
		//
		// Returns the first result row on success, or FALSE on failure.
		// Subsequent rows may be retrieved using select_next() or next_result().
		//
		if ($this->debugqueries) $this->query_debug_start($sql);

		$this->error_code = false;
		$this->log_query($sql);
		$result = @mysql_query($sql,$this->db) or $this->set_error('Could not execute query',$sql,mysql_error($this->db),mysql_errno($this->db));
		if ($result) {
			if ( (!$free_on_return) && ($queryid!==false) ) {
				$this->db_results_stored[$queryid] = $result;
			}
			$this->db_result=$result;
			$this->results = mysql_num_rows($result);

			$res = $this->select_next($no_stripslashes);
			if ($free_on_return) $this->free_result();

			if ($this->debugqueries) $this->query_debug_stop();

			return $res;
		} else {
			if ($this->debugqueries) $this->query_debug_stop();

			return $result; 
		}
	}

	function free_result($queryid = false) {
		$resulthandle = ($queryid!==false) ? $this->db_results_stored[$queryid] : $this->db_result;
		if ($resulthandle) @mysql_free_result($resulthandle);
	}

	function select_next($no_stripslashes=SQL_NO_STRIPSLASHES,$queryid = false) {
		//
		// Wrapper for mysql_fetch_array().
		//
		// Automatically strips escape characters (slashes) from string-type elements
		// prior to returning.
		// Returns the next result row on success, or FALSE on failure.
		//

		$resulthandle = ($queryid!==false) ? $this->db_results_stored[$queryid] : $this->db_result;
		if ($myrow = mysql_fetch_array($resulthandle,$this->fetch_method)) {
			if (!$no_stripslashes) {
				$k = array_keys($myrow);
				for ($i=0; $i<count($myrow); $i++) {
					if (is_string($myrow[$k[$i]])) {
						$myrow[$k[$i]]=stripslashes($myrow[$k[$i]]);
					}
				}
			}
		}
		return $myrow;
	}
	
	function select_all($sql,$byid=false,$groupby=NULL,$groupby_legacy_compatible=true) {
		//
		// Executes a select query and returns ALL result rows for that query.
		// If $byid is true, an associative array is returned, keyed by the row ID.
		//
		// If $groupby is not NULL, rows are grouped by the $groupby column;
		// that is, the result array will contain one or more sub-arrays keyed
		// with the values of the $groupby columns.
		//
		// $groupby_legacy_compatible is for compatibility with older apps, in which
		// results were returned as $res[$groupby_col][0]=$value; disabling compatibility
		// mode returns results as $res[$groupby_col]=$value.
		// 
		
		$output = array();
		$res = $this->select_query($sql,false);
		if (!is_array($res)) { return false; }

		if ($this->debugqueries) $this->query_debug_continue();
		
		$i = 0;
		
		while (is_array($res)) {
			if (!is_null($groupby)) {
				if ($groupby_legacy_compatible) {
					$output[$res[$groupby]][] = $res;
				} else {
					$output[$res[$groupby]] = $res;
				}
			} elseif ($byid) {
				$output[$res["id"]] = $res;
			} else {
				$output[] = $res;
			}
			$res = $this->select_next();
		}
		$this->free_result();

		if ($this->debugqueries) $this->query_debug_stop();
		return $output;
	}

	function next_result() {
		//
		// same as select_next()
		//
		return $this->select_next();
	}
	
	function list_fields($tbl_name) {
		//
		// Returns all fields in the specified table.
		// 
		$fields = mysql_list_fields($this->db_name,$tbl_name,$this->db);
		$columns = mysql_num_fields($fields);

		$res = array();
		for ($i = 0; $i < $columns; $i++) {
	  		$res[] = mysql_field_name($fields, $i);
		}
		return $res;
	}
	
	function timestamp_to_english($timestamp,$shortmonths=false,$dateonly=false,$timeonly=false,$simple=false) {
		//
		// Converts a MySQL timestamp value to an english date/time
		// 
		$mos = array("January","Feburary","March","April","May","June","July","August","September","October","November","December");

		$year = (int) substr($timestamp,0,4);
		$month = (int) substr($timestamp,4,2);
		$day = (int) substr($timestamp,6,2);
		$hour = substr($timestamp,8,2);
		$min = substr($timestamp,10,2);
		$sec = substr($timestamp,12,2);
		
		$mo = $mos[$month-1];
		if ($shortmonths) { $mo = substr($mo,0,3); }
		if ($simple) {
			$date = "$day $mo $year";
			$time = "$hour:$min:$sec";
		} else {
			$date = "$mo $day, $year";
			$time = "$hour:$min:$sec";
		}
		if ($dateonly) { 
			return $date;
		} elseif ($timeonly) {
			return $time;
		} else {
			return "$date $time";
		}
	}
	
	function get_cols_vals($arr,&$cols,&$vals,$safe=true,$restrict_keys=NULL) {
		if (is_array($arr)) {
			reset($arr);
			foreach ($arr as $name=>$value) {
				if (is_array($restrict_keys)) {
					if (!in_array($name,$restrict_keys)) continue;
					if (is_numeric($name)) continue;
				}
				
				$cols .= '`'.$name.'`,';
				if ($safe) {
					$vals .= "'".$this->safestr($value)."',";
				} else {
					$vals .= $value.",";
				}
			}
			$cols = substr($cols,0,strlen($cols)-1);
			$vals = substr($vals,0,strlen($vals)-1);
		}
	}

	function insert_array_query($table,$values,$literals=NULL,$restrict_keys=NULL,$multiple=false) {
		//
		// Inserts values into a table directly from an array
		//
		// $table is the table to insert into; $values is an array of
		// fieldname=>value pairs to insert into the table.
		//
		// $literals is an additional array of fieldname=>value pairs to insert,
		// however "value" is not escaped (so it may contain MySQL commands such
		// as "NOW()", etc.)
		//
		// If $restrict_keys is non-null, it must specify a list of valid field
		// names to pull from values; any unspecified fields will be ignored.
		//
		// If $multiple is TRUE, then $values is interpreted as a multi-
		// dimensional array -- each element in $values should be an array
		// representing a row to insert into the database.
		//
		// Returns non-zero on success, false on failure.
		//
		
		if (!$multiple) $values = array($values);
		
		$rows = array();
		$cols = '';
		$first = true;
		foreach ($values as $k=>$row) {
			$tmpcols = '';
			$vals = '';
			if ($this->stripslashes_before_query) $row = $this->array_stripslashes($row);

			$this->get_cols_vals($row,$tmpcols,$vals,true,$restrict_keys);
			if (is_array($literals)) {
				$this->get_cols_vals($literals,$cols_l,$vals_l,false);
				if ($cols_l) {
					$tmpcols .= ($tmpcols?',':'').$cols_l;
					$vals .= ($vals?',':'').$vals_l;
				}
			}
			$rows[] = '(' . $vals . ')';
			if ($first) {
				$first = false;
				$cols = $tmpcols;
			}
		}
		
		$query = "INSERT INTO $table ($cols) VALUES " . implode(',',$rows);;
		$id = $this->insert_query($query);
		return $id;
	}

	function update_array_query($table,$values,$id,$literals=NULL,$restrict_keys=NULL,$whereclause=NULL) {

		if ($this->stripslashes_before_query) $values = $this->array_stripslashes($values);

		//
		// Updates values in a table directly from an array
		//
		// $table is the table to update; $values is an array of
		// fieldname=>value pairs to insert into the table; $id
		// is the ID number to update.
		//
		// Returns non-zero on success, false on failure.
		// 
		$colvals = '';
		reset($values);
		foreach ($values as $name=>$value) {
			if (is_array($restrict_keys)) {
				if (!in_array($name,$restrict_keys)) continue;
				if (is_numeric($name)) continue;
			}
			$colvals .= '`'.$name.'`="'.$this->safestr($value).'",';
		}
		$colvals = substr($colvals,0,strlen($colvals)-1);
		
		if (is_array($literals)) {
			reset($literals);
			foreach ($literals as $name=>$value) {
				$colvals_l .= '`'.$name.'`='.$value.',';
			}
			if ($colvals_l) {
				$colvals .= ($colvals?',':'').substr($colvals_l,0,strlen($colvals_l)-1);
			}
		}
		
		$query = "UPDATE $table SET $colvals".($id==NULL?"":" WHERE id=".((int) $id)).($whereclause==NULL?"":" WHERE $whereclause");
		return $this->misc_query($query);
	}
	
	function update_request_query($table,$fields,$id,$literals=NULL) {
		$values = array();
		foreach ($fields as $k=>$field) {
			$values[$field] = $_REQUEST[$field];
		}
		return $this->update_array_query($table,$values,$id,$literals);
	}

	function insert_request_query($table,$fields,$extrafields=NULL,$literals=NULL) {
		//
		// Inserts values into a table directly from the $_REQUEST array.
		//
		// $table is the table to insert into; $fields is an array of
		// field names to insert.  Field names MUST match the keys in
		// the $_REQUEST array.
		//
		// Returns non-zero on success, false on failure.
		// 
		reset($fields);
		foreach ($fields as $k=>$name) {
			$cols .= '`'.$name.'`,';
			$vals .= "'".$this->safestr($_REQUEST[$name])."',";
		}
		if (is_array($extrafields)) {
			foreach ($extrafields as $name=>$value) {
				$cols .= '`'.$name.'`,';
				$vals .= "'".$this->safestr($value)."',";
			}			
		}

		$cols = substr($cols,0,strlen($cols)-1);
		$vals = substr($vals,0,strlen($vals)-1);
		
		if (is_array($literals)) {
			$this->get_cols_vals($literals,$cols_l,$vals_l,false);
			if ($cols_l) {
				$cols .= ",$cols_l";
				$vals .= ",$vals_l";
			}
		}
		

		$query = "INSERT INTO $table ($cols) VALUES ($vals)";
		return $this->insert_query($query);
	}
	
	function keys_to_vals(&$arr) {
		$res = array();
		foreach ($arr as $k=>$v) {
			$res[] = $k;
		}
		return $res;
	}
	
	function array_stripslashes($arr) {
		if (!get_magic_quotes_gpc()) return $arr;
		foreach ($arr as $k=>$v) {
			if (is_string($v)) {
				$arr[$k] = stripslashes($v);
			} elseif (is_array($v)) {
				$arr[$k] = $this->array_stripslashes($v);
			}
		}
		return $arr;
	}
	
	function begin() {
		$query = 'BEGIN';
		return $this->misc_query($query);
	}

	function commit() {
		$query = 'COMMIT';
		return $this->misc_query($query);
	}

	function rollback() {
		$query = 'ROLLBACK';
		return $this->misc_query($query);
	}

	/* Lock tables.  A function with really freaky syntax.
	 *
	 * To lock a single table:
	 *	lock('tablename','mode'); // where 'mode' is 'read' or 'write'
	 *
	 * To lock multiple tables:
	 *	lock( array('tablename1'=>'mode1','tablename2'=>'mode2') );
	 *
	 */
	function lock($tables,$mode = 'write') {
		if (!is_array($tables)) $tables = array($tables=>$mode);
		
		$query = "LOCK TABLES ";
		foreach ($tables as $tablename=>$mode) {
			$query .= $tablename." ".$mode.", ";
		}
		
		$query = substr($query,0,strlen($query)-2);
		return $this->misc_query($query);
	}
	
	function unlock() {
		$query = "UNLOCK TABLES";
		return $this->misc_query($query);
	}

	function __sleep() {
		if ($this->connected) {
			$this->close_connection();
			$this->connected = true;
		}
		return array_keys(get_object_vars($this));
	}

	function __wakeup() {
		if ($this->connected) {
			$this->connect_database();
		}
	}
	
	function &singleton($new_name="",$new_host="",$new_user="",$new_pass="") {
		if (!isset($GLOBALS['singletons']['SQL'])) {
			$GLOBALS['singletons']['SQL'] = &new SQL($new_name,$new_host,$new_user,$new_pass);
		}
		
		return $GLOBALS['singletons']['SQL'];
	}

}

?>
